# Wednesday, January 22, 2014, 5:01 AM, VKB
# Notes on mapping EVs into a rectangular coordinate system to further study
# bulb ramping used in timelapse photography
# 
# VALUE FOR x (time)
# In a spreadsheet, a cell would be specified which contains our Time
# for each image's origin expressed for example as G2 but for discussion
# shown here instead for clarity as hh:mm:ss
# 
# Getting the time of day into a value for x
# Value_x=(((((SECOND(hh:mm:ss)/60)+MINUTE(hh:mm:ss))/60)+HOUR(hh:mm:ss)))/24*10000
# 
# Note: There are 86,400 seconds in 24 hours so in the example above our
# integer values for x will be in increments equivalent to 8.64 seconds
# which will be fine with intervals between timelapse frame of 10 seconds
#
# That's fine for OpenOffice Calc, but how about getting this produced by ExifTool?
# And then the same question applies to the other intermediate steps listed below
# for y - how to write this in Perl and get it to work in ExifTool?
# 
# First crack at custom tag hack for: .EV_calculator_01
# Many many thanks to Phil Harvey and ExifTool
#
# The %Image::ExifTool::UserDefined hash defines new tags to be added
# to existing tables.

%Image::ExifTool::UserDefined = (
    # All EXIF tags are added to the Main table, and WriteGroup is used to
    # specify where the tag is written (default is ExifIFD if not specified):
    'Image::ExifTool::Exif::Main' => {
        0xd000 => {
            Name => 'NewEXIFTag',
            Writable => 'int16u',
            WriteGroup => 'IFD0',
        },
        # add more user-defined EXIF tags here...
    },

# Composite tags are added to the Composite table:
  'Image::ExifTool::Composite' => {

      # We don't need the extension of the file, let's strip it off 
         BaseName => {
            Require => {
                 0 => 'FileName',
            },
             # remove the extension from FileName
             ValueConv => '$val[0] =~ /(.*)\./ ? $1 : $val[0]',
        },

      # We'll need the time separated from the date
      # return the time from a date/time string
         Time => {
      	    Require => {
                 0 => 'DateTimeOriginal',
            },
             ValueConv => '$val =~ s/.* //; $val',
      	},

# At this point it's quite murky and I have no idea how to deal with time
# in Perl, or even if it need be treated differently than any other number
# so maybe regex alone will do? ... FWIW, here's a leap into the leaf pile :)
	
     # Getting the time of day into a value for x
	Time_into_x => {
	   Require => {
	   	0 = 'Time',
	   },
	  # determine Time's fractional part of day and multiply by 10000
	  #(((((ss/60)+mm)/60)+hh))/24*10000
	    ValueConv => '$val =~ to be determined ',
	},

# VALUE FOR y (EV corrected for ISO)
# In a spreadsheet, a cell would be specified which contains our ShutterSpeed
# and in another cell, Aperture for each image expressed for example as D2 and
# C2 but for discussion shown here instead for clarity as t and N respectively.
# 
# First the EV_base value (otherwise displayed EV subscript 100)
# EV_base=LOG10(((N*N)/t))/LOG10(2)
#
# In a spreadsheet, a cell would be specified which contains our ISO value for
# each image expressed for example as E2 but for discussion shown here instead 
# for clarity as ISO
# 
# Second step is to calculate EV_ISO
# EV_ISO=EV_base-LOG10((ISO/100))/LOG10(2)
# 
# Note: As of this writing, the Wikipedia page showing this calculation has the
# wrong algebraic sign. The main active author of that article has been notified.
# http://en.wikipedia.org/wiki/Exposure_value
#
# Further reading at photo.stackexchange found a much cleaner and clearer formula
# http://photo.stackexchange.com/questions/32359/why-does-ev-increase-as-iso-increases
# written by Matthew Miller in answer to the user's question:
# 
# EV = log2(N) + log2(1/t) - log2(100/S)
#
# And more tersely stated:
#
# EV = log2(N/(t(S/100)))
# 
# For N we'll use 'aperturevalue', for t 'shutterspeedvalue' and for S 'ISO'

	# First let's define EV
	EV => {
	   Require => {
	   	0 = 'aperturevalue',
	   	1 = 'shutterspeedvalue',
	   	2 = 'ISO',
	},

# Again, I have no idea how to deal with doing this calculation with ExifTool and base 2 logs
# in Perl, or even if it's practical to consider doing so, but no harm in trying :)

	# We'll use Matt's shorter formula: EV = log2(N/(t(S/100))) and adapt it to our tags:
	# EV=(LOG10((aperturevalue*aperturevalue)/(shutterspeedvalue*(ISO/100))))/LOG10(2)
	  ValueConv => '$val = to be determined ',
     },  



# Now we can heighten or flatten the mapped EV curve for any given set of images as much as
# you like with a multiplier, shown here by a factor of 100 for the y coordinate 

	EV_into_y => {
	   Require => {
	   	0 = 'EV',
	},
	  ValueConv => '$val = EV*100',
    },

);